@@ -1,3 +1,5 @@
+import time
+import queue
 from state_machine_py.request import Request
 
 
@@ -37,6 +39,7 @@ class StateMachine():
         self._is_terminate = False  # 永遠に停止
         self._intermachine = intermachine
         self._name = name
+        self._input_queue = queue.Queue()
 
     @property
     def context(self):
@@ -58,16 +61,6 @@ class StateMachine():
         """現在の辺"""
         return self._edge_path
 
-    @property
-    def lines_getter(self):
-        """このステートマシンは、このContextが何なのか知りません。
-        外部から任意に与えることができる変数です"""
-        return self._lines_getter
-
-    @lines_getter.setter
-    def lines_getter(self, val):
-        self._lines_getter = val
-
     @property
     def verbose(self):
         """標準出力にデバッグ情報を出力するなら真"""
@@ -82,69 +75,151 @@ class StateMachine():
         """永遠に停止"""
         return self._is_terminate
 
-    @is_terminate.setter
-    def is_terminate(self, val):
-        self._is_terminate = val
+    @property
+    def input_queue(self):
+        """入力キュー"""
+        return self._input_queue
 
     @property
     def name(self):
         """他のステートマシンと区別するためのキーとして使われます"""
         return self._name
 
+    def terminate(self):
+        """ステートマシンを終了させます"""
+        if self.verbose:
+            print(
+                f"{self._alternate_state_machine_name()} Terminate")
+
+        self._is_terminate = True
+
+        req = Request(
+            context=self._context,
+            edge_path=self.edge_path,
+            intermachine=self._intermachine)
+        self.on_terminate(req)
+
     def start(self, next_state_name):
-        """まず state_machine._arrive(...) を行い、
-        そのあと _leave(...), _arrive(...) のペアを無限に繰り返します。
-        _leave(...) に渡す line 引数は _arrive(...) から返しますが、
-        代わりに None を返すと self.lines_getter() が実行されます。
-        self.lines_getter() は、 line のリストを返す関数です。
-        self.lines_getter() が None を返すとループを抜けます
-        """
-        if self._is_terminate:
-            return
+        """ステートマシンを開始します"""
 
-        # Arrive と Leave はペアになります
-        # Leave が終わったところで IsTerminate を判定します
-
-        # Arrive sequence
-        interrupt_line = self._arrive(next_state_name)
-
-        while interrupt_line:
-            next_state_name = self._leave(interrupt_line)
-            if self._is_terminate:
-                return
-
-            interrupt_line = self._arrive(next_state_name)
-        #
-
-        # Leave and loop
-        if self._is_terminate:
-            return
+        # [Arrive] --> [Get] --> [Leave] を最小単位とするループです。
+        # しかしコードは [Get] --> [Leave] --> [Arrive] になってしまっているので
+        # スタート直後の１回だけ [Get] --> [Leave] をスキップしてください
+        is_skip_leave = True
 
+        # 無限ループ
         while True:
-            lines = self.lines_getter()
-            if lines is None:
-                break
 
-            for line in lines:
+            # ステートマシンの終了のタイミングの１つ目です。 ループの先頭で終了させます
+            if self._is_terminate:
+                if self.verbose:
+                    print(
+                        f"{self._alternate_state_machine_name()} Terminate the state machine (Loop A begin)")
+                return  # start関数を終わります
 
-                self.on_line(line)
+            if self.verbose:
+                print(
+                    f"{self._alternate_state_machine_name()} Loop(A) Begin next_state_name={next_state_name}")
 
-                next_state_name = self._leave(line)
+            # 次のループの初回だけ、無条件に通ります
+            is_enter_loop = True
+
+            # キューに内容がある間、繰り返します
+            while is_enter_loop or not self._input_queue.empty:
+
+                # ステートマシンの終了のタイミングの２つ目です。ループの先頭で終了させます
                 if self._is_terminate:
-                    return
+                    if self.verbose:
+                        print(
+                            f"{self._alternate_state_machine_name()} Terminate the state machine (Loop B begin)")
+                    return  # start関数を終わります
 
-                # Arrive sequence
+                is_enter_loop = False
 
-                interrupt_line = self._arrive(next_state_name)
+                if self.verbose:
+                    print(
+                        f"{self._alternate_state_machine_name()} Loop(B) Begin")
 
-                while interrupt_line:
-                    next_state_name = self._leave(interrupt_line)
-                    if self._is_terminate:
-                        return
+                if is_skip_leave:
+                    # 初回の Leave をスキップしました
+                    if self.verbose:
+                        print(
+                            f"{self._alternate_state_machine_name()} Passed first leave")
+                    is_skip_leave = False
+                else:
+                    if self.verbose:
+                        print(
+                            f"{self._alternate_state_machine_name()} Wait GetQueueline")
 
-                    interrupt_line = self._arrive(next_state_name)
+                    # queue の get() でブロックするとデッドロックしてしまう
+                    line = None
+                    while line is None:
+                        # ステートマシンの終了のタイミングの３つ目です。ループの先頭で終了させます
+                        if self._is_terminate:
+                            if self.verbose:
+                                print(
+                                    f"{self._alternate_state_machine_name()} Terminate the state machine (Loop C begin)")
+                            return  # start関数を終わります
+
+                        try:
+                            # +-------+
+                            # |       |
+                            # |  Get  |
+                            # |       |
+                            # +-------+
+                            line = self._input_queue.get(
+                                block=False)
+                            self._input_queue.task_done()
+                        except queue.Empty:
+                            line = None
+
+                        time.sleep(0)
+
+                    if self.verbose:
+                        print(
+                            f"{self._alternate_state_machine_name()} GetQueueline={line}")
+
+                    # +-------+
+                    # |       |
+                    # | Leave |
+                    # |       |
+                    # +-------+
+                    next_state_name = self._leave(line)
+
+                    if self.verbose:
+                        print(
+                            f"{self._alternate_state_machine_name()} After leave next_state_name={next_state_name}")
+
+                    # ステートマシンの終了のタイミングの４つ目です。ループの先頭で終了させます
+                    if next_state_name is None:
+                        self.terminate()
+
+                        if self.verbose:
+                            print(
+                                f"{self._alternate_state_machine_name()} Terminate the state machine (After leave)")
+                        return  # start関数を終わります
+
+                if self.verbose:
+                    print(
+                        f"{self._alternate_state_machine_name()} Before arrive next_state_name={next_state_name}")
+
+                # ループの初回はここから始まります
                 #
-        #
+                # +--------+
+                # |        |
+                # | Arrive |
+                # |        |
+                # +--------+
+                self._arrive(next_state_name)
+
+            if self.verbose:
+                print(
+                    f"{self._alternate_state_machine_name()} Queue is empty")
+
+            # このままでは いつまでも ここを通るので 少し待ってみます
+            time.sleep(0.016)  # TODO スリープタイムを設定できたい
+
+            # ここまでが１つの処理です
 
     def _alternate_state_machine_name(self):
         if self.name is None:
@@ -160,18 +235,10 @@ class StateMachine():
         ----------
         str : next_state_name
             次の状態の名前
-
-        Returns
-        -------
-        object
-            ただちに _leave に渡したい引数。無ければ None
         """
 
-        if self._is_terminate:
-            return
-
         if self.verbose:
-            edge_path = '.'.join(self._edge_path)
+            edge_path = '/'.join(self._edge_path)
             print(
                 f"{self._alternate_state_machine_name()} Arrive to {next_state_name} {edge_path}")
 
@@ -183,12 +250,8 @@ class StateMachine():
                           edge_path=self._edge_path,
                           line=None,
                           intermachine=self._intermachine)
-            interrupt_line = self._state.entry(req)
-            if interrupt_line and self.verbose:
-                print(
-                    f"{self._alternate_state_machine_name()} Arrive interrupt_line={interrupt_line}")
 
-            return interrupt_line
+            self._state.entry(req)
 
         else:
             # Error
@@ -209,8 +272,6 @@ class StateMachine():
         str
             次の状態の名前
         """
-        if self._is_terminate:
-            return
 
         if self.verbose:
             print(
@@ -247,8 +308,7 @@ class StateMachine():
         if self.state.name in self._transition_dict:
             curr_dict = self._transition_dict[self.state.name]
             if curr_dict is None:
-                self._is_terminate = True
-                self.on_terminate(req)
+                self.terminate()
                 return
         else:
             raise ValueError(
@@ -259,8 +319,7 @@ class StateMachine():
             if edge in curr_dict:
                 curr_dict = curr_dict[edge]
                 if curr_dict is None:
-                    self._is_terminate = True
-                    self.on_terminate(req)
+                    self.terminate()
                     return
             else:
                 raise ValueError(
